local _
local _G = _G
local addonName, LFGShout = ...
local VERSION = GetAddOnMetadata(addonName, "Version")

-- =============================================================================================================

local LFGShoutFrame = CreateFrame("Frame", "LFGShoutFrame", UIParent)

-- =============================================================================================================

local DUNGEON_LIST = {
	{ Name = "Ragefire Chasm",		Code = "RFC",						Level = "13 - 18" },
	{ Name = "Wailing Caverns",		Code = "WC",						Level = "17 - 24" },
	{ Name = "The Deadmines",			Code = "Dead Mines",		Level = "17 - 26" },
	{ Name = "Shadowfang Keep",		Code = "SFK",						Level = "22 - 30" },
	{ Name = "Blackfathom Deeps", Code = "BFD",						Level = "24 - 32" },
	{ Name = "The Stockade",			Code = "Stockade",			Level = "24 - 32" },
	{ Name = "Gnomeregan",				Code = "Gnomeregan",		Level = "29 - 38" },
	{ Name = "Razorfen Kraul",		Code = "RFK",						Level = "29 - 38" },
	{ Name = "SM - Graveyard",		Code = "SM GY",					Level = "26 - 36" },
	{ Name = "SM - Library",			Code = "SM Lib",				Level = "29 - 39" },
	{ Name = "SM - Armory",				Code = "SM Armory",			Level = "32 - 42" },
	{ Name = "SM - Cathedral",		Code = "SM Cath",				Level = "35 - 45" },
	{ Name = "Razorfen Downs",		Code = "RFD",						Level = "37 - 46" },
	{ Name = "Uldaman",						Code = "ULDA",					Level = "41 - 51" },
	{ Name = "Zul'Farrak",				Code = "ZF",						Level = "42 - 46" },
	{ Name = "Maraudon",					Code = "MARA",					Level = "46 - 55" },
	{ Name = "Sunken Temple",			Code = "ST",						Level = "50 - 56" },
	{ Name = "Blackrock Depths",	Code = "BRD",						Level = "52 - 60" },
	{ Name = "Blackrock Spire L", Code = "LBRS",					Level = "55 - 60" },
	{ Name = "Blackrock Spire U", Code = "UBRS",					Level = "55 - 60" },
	{ Name = "Dire Maul East",		Code = "DM East",				Level = "55 - 60" },
	{ Name = "Dire Maul West",		Code = "DM West",				Level = "55 - 60" },
	{ Name = "Dire Maul North",		Code = "DM North",			Level = "55 - 60" },
	{ Name = "Scholomance",				Code = "Scholo",				Level = "55 - 60" },
	{ Name = "Stratholme Live",		Code = "Strat Live",		Level = "55 - 60" },
	{ Name = "Stratholme Undead",	Code = "Strat Undead",	Level = "55 - 60" },
}

local CHANNEL_NAMES = {
	["enGB"] = { GENERAL = "General - ", TRADE = "Trade - City", LFG = "LookingForGroup", WORLD = "World" },
	["enUS"] = { GENERAL = "General - ", TRADE = "Trade - City", LFG = "LookingForGroup", WORLD = "World" },
	["deDE"] = { GENERAL = "Allgemein - ", TRADE = "Handel", LFG = "SucheNachGruppe", WORLD = "Weltverteidigung" },
	["esES"] = { GENERAL = "General - ", TRADE = "Trade - City", LFG = "LookingForGroup", WORLD = "World" },
	["esMX"] = { GENERAL = "General - ", TRADE = "Trade - City", LFG = "LookingForGroup", WORLD = "World" },
	["frFR"] = { GENERAL = "General - ", TRADE = "Trade - City", LFG = "LookingForGroup", WORLD = "World" },
	["koKR"] = { GENERAL = "General - ", TRADE = "Trade - City", LFG = "LookingForGroup", WORLD = "World" },
	["ptBR"] = { GENERAL = "General - ", TRADE = "Trade - City", LFG = "LookingForGroup", WORLD = "World" },
	["ruRU"] = { GENERAL = "Общий: ", TRADE = "Торговля: Город", LFG = "ПоискСпутников", WORLD = "World" },
	["zhCN"] = { GENERAL = "综合 - ", TRADE = "交易 - 城市", LFG = "寻找组队", WORLD = "世界频道" },
	["zhTW"] = { GENERAL = "General - ", TRADE = "Trade - City", LFG = "LookingForGroup", WORLD = "World" },
}

local locale = GetLocale()
local GENERAL_CHANNEL_NAME = CHANNEL_NAMES[locale].GENERAL
local TRADE_CHANNEL_NAME = CHANNEL_NAMES[locale].TRADE
local LFG_CHANNEL_NAME = CHANNEL_NAMES[locale].LFG
local WORLD_CHANNEL_NAME = CHANNEL_NAMES[locale].WORLD

local DUNGEON_BUTTONS = { }
local GENERAL_CHANNEL, TRADE_CHANNEL, LFG_CHANNEL, WORLD_CHANNEL = 0, 0, 0, 0
local LEFT_BUTTON, RIGHT_BUTTON, MIDDLE_BUTTON = "LeftButton", "RightButton", "MiddleButton"
local MIN_HEIGHT, MAX_HEIGHT = 22, 19 * #DUNGEON_LIST
local MARKER = "|T1121272:16:16:0:0:512:512:202:234:440:472|t"

-- =============================================================================================================
	
local function LFGShout_SaveSettings()
	LFGShout_Settings.StartText = LFGShoutFrame.OptionsFrame.StartText:GetText()
	LFGShout_Settings.EndText = LFGShoutFrame.OptionsFrame.EndText:GetText()
	LFGShout_Settings.ChannelGeneral = LFGShoutFrame.OptionsFrame.ChannelGeneral:GetChecked()
	LFGShout_Settings.ChannelTrade = LFGShoutFrame.OptionsFrame.ChannelTrade:GetChecked()
	LFGShout_Settings.ChannelLFG = LFGShoutFrame.OptionsFrame.ChannelLFG:GetChecked()
	LFGShout_Settings.ChannelWorld = LFGShoutFrame.OptionsFrame.ChannelWorld:GetChecked()
	LFGShout_Settings.ChannelGuild = LFGShoutFrame.OptionsFrame.ChannelGuild:GetChecked()
	LFGShout_Settings.ChannelSay = LFGShoutFrame.OptionsFrame.ChannelSay:GetChecked()
	LFGShout_Settings.ChannelYell = LFGShoutFrame.OptionsFrame.ChannelYell:GetChecked()
	wipe(LFGShout_Settings.CheckedDungeons)
	for i, child in pairs(DUNGEON_BUTTONS) do
		if child:GetChecked() then
			tinsert(LFGShout_Settings.CheckedDungeons, child:GetID())
		end
	end
	--print("LFGShout_Settings: SAVE")
end

-- =============================================================================================================

local function LFGShout_Shout(button)
	local text = format("%s%s: ", LFGShout_Settings.PrefixStar and "{rt1} " or "", LFGShoutFrame.OptionsFrame.StartText:GetText())
	for i, child in pairs(DUNGEON_BUTTONS) do
		if child:GetChecked() then 
			text = format("%s%s / ", text, DUNGEON_LIST[child:GetID()].Code)
		end
	end
	text = format("%s  %s", strtrim(text, " / "), LFGShoutFrame.OptionsFrame.EndText:GetText())

	if IsModifierKeyDown() then --if button == LEFT_BUTTON then
		print(format("%s|c00FF5C1ALFGShout: |r%s", MARKER, text))
	else
		if LFGShoutFrame.OptionsFrame.ChannelGuild:GetChecked() then
			SendChatMessage(text, "GUILD")
		end

		if LFGShoutFrame.OptionsFrame.ChannelSay:GetChecked() then
			SendChatMessage(text, "SAY")
		end

		if LFGShoutFrame.OptionsFrame.ChannelYell:GetChecked() then
			SendChatMessage(text, "YELL")
		end

		--local timeout = 0.1

		if LFGShoutFrame.OptionsFrame.ChannelGeneral:GetChecked() then
			--C_Timer.After(timeout, function() 
			SendChatMessage(text, "CHANNEL", nil, GENERAL_CHANNEL)
			--end)
			--timeout = timeout + 0.1
		end
		
		if LFGShoutFrame.OptionsFrame.ChannelTrade:GetChecked() then
			--C_Timer.After(timeout, function() 
			SendChatMessage(text, "CHANNEL", nil, TRADE_CHANNEL)
			--end)
			--timeout = timeout + 0.1
		end
		
		if LFGShoutFrame.OptionsFrame.ChannelLFG:GetChecked() then
			--C_Timer.After(timeout, function() 
			SendChatMessage(text, "CHANNEL", nil, LFG_CHANNEL)
			--end)
			--timeout = timeout + 0.1
		end

		if LFGShoutFrame.OptionsFrame.ChannelWorld:GetChecked() then
			--C_Timer.After(timeout, function() 
			SendChatMessage(text, "CHANNEL", nil, WORLD_CHANNEL)
			--end)
		end

	end
end

-- =============================================================================================================

local function LFGShout_JoinChannel(channel, channelName)
	if not channel or channel < 1 then
		print(format("|c00FF5C1ALFGShout: |rJoining channel: %s", channelName))
		JoinChannelByName(channelName)
		ChatFrame_AddChannel(DEFAULT_CHAT_FRAME, channelName)
	end
end

-- =============================================================================================================

local function LFGShout_RefreshChannels()
	GENERAL_CHANNEL = GetChannelName(format("%s%s", GENERAL_CHANNEL_NAME, GetRealZoneText() or ""))
	TRADE_CHANNEL = GetChannelName(TRADE_CHANNEL_NAME)
	LFG_CHANNEL = GetChannelName(LFG_CHANNEL_NAME)
	WORLD_CHANNEL = GetChannelName(WORLD_CHANNEL_NAME)
--	LFGShoutFrame.OptionsFrame.ChannelGeneral.Text:SetText("General: " .. GENERAL_CHANNEL)
--	LFGShoutFrame.OptionsFrame.ChannelTrade.Text:SetText("Trade: " .. TRADE_CHANNEL)
--	LFGShoutFrame.OptionsFrame.ChannelLFG.Text:SetText("LookingForGroup: " .. LFG_CHANNEL)
	LFGShoutFrame.OptionsFrame.ChannelGeneral.Text:SetFormattedText("|c00FF5C1A%d|r General", GENERAL_CHANNEL or 0)
	LFGShoutFrame.OptionsFrame.ChannelTrade.Text:SetFormattedText("|c00FF5C1A%d|r Trade", TRADE_CHANNEL or 0)
	LFGShoutFrame.OptionsFrame.ChannelLFG.Text:SetFormattedText("|c00FF5C1A%d|r LookingForGroup", LFG_CHANNEL or 0)
	LFGShoutFrame.OptionsFrame.ChannelWorld.Text:SetFormattedText("|c00FF5C1A%d|r World", WORLD_CHANNEL or 0)
	--print("|c00FF5C1ALFGShout: |rRefreshing Channels.")
end

-- =============================================================================================================

--local function LFGShout_Minimize(self)
----	self:SetHeight(self:GetHeight() < 30 and MAX_HEIGHT or MIN_HEIGHT)
--	self:SetHeight(MIN_HEIGHT)
--	for i, child in pairs(DUNGEON_BUTTONS) do
--		child:SetShown(false)
--	end
----	self.OptionsFrame:SetShown(false)
--end

-- =============================================================================================================

--local function LFGShout_Maximize(self)
--	self:SetHeight(MAX_HEIGHT)
--	for i, child in pairs(DUNGEON_BUTTONS) do
--		child:SetShown(true)
--	end
--end

-- =============================================================================================================

local function LFGShout_Load()

	if LFGShout_Settings == nil then
		LFGShout_Settings = { }
		LFGShout_Settings.CheckedDungeons = { }
	end

	-- Main Frame ------------------------------------------------------------------------

	LFGShoutFrame:SetFrameLevel(20)
	LFGShoutFrame:SetMovable(true)
	LFGShoutFrame:EnableMouse(true)
	LFGShoutFrame:SetClampedToScreen(true)
	LFGShoutFrame:SetSize(145, MAX_HEIGHT)
	LFGShoutFrame:RegisterForDrag(LEFT_BUTTON)
	LFGShoutFrame:SetBackdrop({
		bgFile="Interface/RaidFrame/UI-RaidFrame-GroupBg",
		edgeFile="Interface/Tooltips/UI-Tooltip-Border", 
		tile = false, tileSize = 0, edgeSize = 10,
		insets = { left = 2, right = 2, top = 2, bottom = 2 }
	})
	LFGShoutFrame:SetBackdropColor(0, 0, 0, 0.7)
	LFGShoutFrame:SetBackdropBorderColor(0.1, 0.1, 0.1)
	LFGShoutFrame:SetScript("OnDragStart", function(self, button) self:StartMoving() end)
	LFGShoutFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() LFGShoutFrame.OptionsFrame:Reposition() end)
--	LFGShoutFrame:SetScript("OnEnter", function(self) self:SetAlpha(1) end)
--	LFGShoutFrame:SetScript("OnLeave", function(self) C_Timer.After(0.5, function() self:SetAlpha(0.5) end) end)

--	LFGShoutFrame:SetScript("OnEnter",function(self) LFGShout_Maximize(self) end)
--	LFGShoutFrame:SetScript("OnLeave", function(self, motion) self:SetHeight(MIN_HEIGHT) end)
--	LFGShoutFrame:SetScript("OnLeave", function(self) C_Timer.After(1, function() LFGShout_Minimize(self) end) end)

	-- Check Buttons
	local button
	for i, info in pairs(DUNGEON_LIST) do
		button = CreateFrame("CheckButton", nil, LFGShoutFrame, "InterfaceOptionsCheckButtonTemplate")
		LFGShoutFrame["DungeonCheckButton" .. i] = button
		button:SetPoint("TOPLEFT", 3, (button:GetHeight()-18 * i) - 25)
		button:SetHitRectInsets(0, -110, 1, 5)
		button:SetFrameStrata("MEDIUM")
		--button.Text:SetTextColor(1, 0.82, 0)
--		button.Text:SetText(format("%s |c00999999(%s)|r", info.Name, info.Level))
		button.Text:SetText(info.Name)
		button:SetID(i)
		button:SetChecked(tContains(LFGShout_Settings.CheckedDungeons, button:GetID()))
		button:SetScript("OnEnter", 
			function(self, motion)
				self.Text:SetTextColor(1.00, 0.35, 0.00)
				local point, _, _, x = LFGShoutFrame:GetPoint()
				if string.find(point, "LEFT") and x < 205 then
					GameTooltip:SetOwner(self, "ANCHOR_NONE")
					GameTooltip:SetPoint("LEFT", self, "RIGHT", 100, 0)
				else
					GameTooltip:SetOwner(self, nil, -5, -30)
				end
				GameTooltip:SetText(info.Name)
				GameTooltipTextRight1:SetText(self:GetID())
				GameTooltipTextRight1:SetTextColor(1, 1, 1, 0.1)
				GameTooltipTextRight1:Show()
				GameTooltip:AddDoubleLine(info.Code, info.Level, 1, 1, 1, 1, 1, 1)
				--GameTooltip:AddDoubleLine("ID", self:GetID(), 0.2, 0.2, 0.2, 0.2, 0.2, 0.2)
				GameTooltip:Show()
			end
		)
		button:SetScript("OnLeave", function(self, motion) self.Text:SetTextColor(1, 1, 1) GameTooltip:Hide() end)
		button:SetScript("OnClick", function(self, button) LFGShout_SaveSettings() PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON) end)
		tinsert(DUNGEON_BUTTONS, button)
	end


	LFGShoutFrame:SetScript("OnMouseUp", 
		function(self, button, down)
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
			if button == RIGHT_BUTTON then
				self:SetHeight(self:GetHeight() < 30 and MAX_HEIGHT or MIN_HEIGHT)
				for i, child in pairs(DUNGEON_BUTTONS) do
					child:SetShown(not child:IsShown())
				end
				--LFGShoutFrame.OptionsFrame:SetShown(false)
			end
			if button == MIDDLE_BUTTON then
				LFGShoutFrame.Icon:SetShown(not LFGShoutFrame.Icon:IsShown())
				LFGShout_Settings.IconVisible = LFGShoutFrame.Icon:IsShown()
			end
			LFGShout_Settings.MainFrameVisible = math.floor(LFGShoutFrame:GetHeight()) > MIN_HEIGHT
		end
	)

	-- Title
	LFGShoutFrame.Title = LFGShoutFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	LFGShoutFrame.Title:SetPoint("TOP", 0, -5)
	LFGShoutFrame.Title:SetText(addonName)

	-- Icon on top
	LFGShoutFrame.Icon = LFGShoutFrame:CreateTexture(nil, "ARTWORK")
--	LFGShoutFrame.Icon:SetTexture("Interface/WORLDSTATEFRAME/OrcHead.blp")
--	LFGShoutFrame.Icon:SetPoint("TOP", 0, 26)

	SetPortraitTexture(LFGShoutFrame.Icon, "player")
	LFGShoutFrame.Icon:SetPoint("TOP", 0, 110)
	LFGShoutFrame.Icon:SetScale(0.2)
--	LFGShoutFrame.Icon:SetSize(30, 30)


--	LFGShoutFrame.Icon = LFGShoutFrame:CreateTexture(nil, "BACKGROUND")
----	LFGShoutFrame.Icon:SetTexture("Interface/ENCOUNTERJOURNAL/UI-EJ-BOSS-Glazer.blp")
--	LFGShoutFrame.Icon:SetTexture("Interface/ENCOUNTERJOURNAL/UI-EJ-BOSS-Muru.blp")
----	LFGShoutFrame.Icon:SetTexture("Interface/ENCOUNTERJOURNAL/UI-EJ-BOSS-Occularus.blp")
----	LFGShoutFrame.Icon:SetTexture("Interface/ENCOUNTERJOURNAL/UI-EJ-BOSS-Captain Cookie.blp")
----	LFGShoutFrame.Icon:SetTexture("Interface/ENCOUNTERJOURNAL/XXXXXXXXXXXX.blp")
----	LFGShoutFrame.Icon:SetSize(40, 40)
--	LFGShoutFrame.Icon:SetPoint("TOP", 0, 61)

	-- Shout Button
	LFGShoutFrame.ShoutButton = CreateFrame("Button", nil, LFGShoutFrame)--, "UIMenuButtonStretchTemplate")
	LFGShoutFrame.ShoutButton:SetPoint("TOPLEFT", 2, 0)
	LFGShoutFrame.ShoutButton:SetSize(20, 20)
	LFGShoutFrame.ShoutButton:SetNormalTexture("Interface/Buttons/UI-GuildButton-MOTD-Disabled.blp")
	LFGShoutFrame.ShoutButton:SetHighlightTexture("Interface/Buttons/UI-GuildButton-MOTD-Up.blp")

	LFGShoutFrame.ShoutButton:SetScript("OnMouseDown", 
		function(self, button, down)
			--self:GetHighlightTexture():SetPoint("TOPLEFT", 2, -2)
			self:SetPoint("TOPLEFT", 4, -1)
			self:SetScale(0.85)
		end
	)

	LFGShoutFrame.ShoutButton:SetScript("OnMouseUp", 
		function(self, button)
			--self:GetHighlightTexture():SetPoint("TOPLEFT", 0, 0)
			self:SetPoint("TOPLEFT", 2, 0)
			self:SetScale(1)
			LFGShout_Shout(button)
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
		end
	)

	-- Options Button
	LFGShoutFrame.OptionsButton = CreateFrame("Button", nil, LFGShoutFrame)--, "UIMenuButtonStretchTemplate")
	LFGShoutFrame.OptionsButton:SetPoint("TOPRIGHT", 0, 0)
	LFGShoutFrame.OptionsButton:SetSize(20, 20)
	LFGShoutFrame.OptionsButton:SetNormalTexture("Interface/GossipFrame/HealerGossipIcon.blp")
	LFGShoutFrame.OptionsButton:SetHighlightTexture("Interface/Buttons/UI-OptionsButton.blp")
--	LFGShoutFrame.OptionsButton:SetHighlightTexture("Interface/Buttons/AdventureGuideMicrobuttonAlert.blp")

	LFGShoutFrame.OptionsButton:SetScript("OnMouseDown", 
		function(self, button, down)
			--self:GetHighlightTexture():SetPoint("TOPRIGHT", 2, -2)
			self:SetPoint("TOPRIGHT", -2, -1)
			self:SetScale(0.85)
		end
	)

	LFGShoutFrame.OptionsButton:SetScript("OnMouseUp", 
		function(self, button)
			--self:GetHighlightTexture():SetPoint("TOPRIGHT", 0, 0)
			self:SetPoint("TOPRIGHT", 0, 0)
			self:SetScale(1)
			LFGShoutFrame.OptionsFrame:Reposition()
			LFGShoutFrame.OptionsFrame:SetShown(not LFGShoutFrame.OptionsFrame:IsShown())
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
			LFGShout_Settings.OptionsFrameVisible = LFGShoutFrame.OptionsFrame:IsShown()
		end
	)

	-- Options Frame ------------------------------------------------------------------------
	LFGShoutFrame.OptionsFrame = CreateFrame("Frame", nil, LFGShoutFrame)
--	LFGShoutFrame.OptionsFrame:SetSize(145, 235)
	LFGShoutFrame.OptionsFrame:SetSize(145, 205)
	LFGShoutFrame.OptionsFrame:SetFrameLevel(18)
	LFGShoutFrame.OptionsFrame:SetClampedToScreen(true)
	LFGShoutFrame.OptionsFrame:SetPoint("TOPLEFT", LFGShoutFrame, "TOPRIGHT", 0, 0)
--	LFGShoutFrame.OptionsFrame:SetPoint("TOPLEFT", LFGShoutFrame, "BOTTOMLEFT", 0, 0)

	-- Reposition Function
	LFGShoutFrame.OptionsFrame.Reposition =
		function()
			LFGShoutFrame.OptionsFrame:ClearAllPoints()
			local offset = select(4, LFGShoutFrame:GetPoint()) + LFGShoutFrame.OptionsFrame:GetWidth()
			--print(offset)
			if offset > 0 and offset <= LFGShoutFrame.OptionsFrame:GetWidth() then
				LFGShoutFrame.OptionsFrame:SetPoint("TOPRIGHT", LFGShoutFrame, "TOPLEFT", 0, 0)
			else
				LFGShoutFrame.OptionsFrame:SetPoint("TOPLEFT", LFGShoutFrame, "TOPRIGHT", 0, 0)
			end
		end
		C_Timer.After(1.5, LFGShoutFrame.OptionsFrame.Reposition)

	LFGShoutFrame.OptionsFrame:SetBackdrop({
		bgFile="Interface/RaidFrame/UI-RaidFrame-GroupBg",
		edgeFile="Interface/Tooltips/UI-Tooltip-Border", 
		tile = false, tileSize = 0, edgeSize = 10,
		insets = { left = 2, right = 2, top = 2, bottom = 2 }
	})
	LFGShoutFrame.OptionsFrame:SetBackdropColor(0, 0, 0, 0.7)
	LFGShoutFrame.OptionsFrame:SetBackdropBorderColor(0.1, 0.1, 0.1)
	--LFGShoutFrame.OptionsFrame:Hide()

	-- General Channel (1) Checkbutton
	LFGShoutFrame.OptionsFrame.ChannelGeneral = CreateFrame("CheckButton", nil, LFGShoutFrame.OptionsFrame, "InterfaceOptionsCheckButtonTemplate")
	LFGShoutFrame.OptionsFrame.ChannelGeneral.Text:SetTextColor(1, 0.82, 0)
	LFGShoutFrame.OptionsFrame.ChannelGeneral:SetPoint("TOPLEFT", 3, 0)
	LFGShoutFrame.OptionsFrame.ChannelGeneral:SetHitRectInsets(0, -110, 1, 5)
	LFGShoutFrame.OptionsFrame.ChannelGeneral:SetChecked(LFGShout_Settings.ChannelGeneral or false)
	LFGShoutFrame.OptionsFrame.ChannelGeneral:SetScript("OnClick", function(self, button) LFGShout_SaveSettings() PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON) end)
	
	-- Trade Channel (2) Checkbutton
	LFGShoutFrame.OptionsFrame.ChannelTrade = CreateFrame("CheckButton", nil, LFGShoutFrame.OptionsFrame, "InterfaceOptionsCheckButtonTemplate")
	LFGShoutFrame.OptionsFrame.ChannelTrade.Text:SetTextColor(1, 0.82, 0)
	LFGShoutFrame.OptionsFrame.ChannelTrade:SetPoint("TOPLEFT", 3, -18)
	LFGShoutFrame.OptionsFrame.ChannelTrade:SetHitRectInsets(0, -110, 1, 5)
	LFGShoutFrame.OptionsFrame.ChannelTrade:SetChecked(LFGShout_Settings.ChannelTrade or false)
	LFGShoutFrame.OptionsFrame.ChannelTrade:SetScript("OnClick", function(self, button) LFGShout_SaveSettings() PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON) end)

	-- LFG Channel (4) Checkbutton
	LFGShoutFrame.OptionsFrame.ChannelLFG = CreateFrame("CheckButton", nil, LFGShoutFrame.OptionsFrame, "InterfaceOptionsCheckButtonTemplate")
	LFGShoutFrame.OptionsFrame.ChannelLFG.Text:SetTextColor(1, 0.82, 0)
	LFGShoutFrame.OptionsFrame.ChannelLFG:SetPoint("TOPLEFT", 3, -36)
	LFGShoutFrame.OptionsFrame.ChannelLFG:SetHitRectInsets(0, -110, 1, 5)
	LFGShoutFrame.OptionsFrame.ChannelLFG:SetChecked(LFGShout_Settings.ChannelLFG or false)
	LFGShoutFrame.OptionsFrame.ChannelLFG:SetScript("OnClick", 
		function(self, button)
			if self:GetChecked() then -- Autojoin LFG channel
				LFGShout_JoinChannel(LFG_CHANNEL, LFG_CHANNEL_NAME)
			end
			LFGShout_SaveSettings()
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
		end)

	-- World Channel (4) Checkbutton
	LFGShoutFrame.OptionsFrame.ChannelWorld = CreateFrame("CheckButton", nil, LFGShoutFrame.OptionsFrame, "InterfaceOptionsCheckButtonTemplate")
	LFGShoutFrame.OptionsFrame.ChannelWorld.Text:SetTextColor(1, 0.82, 0)
	LFGShoutFrame.OptionsFrame.ChannelWorld:SetPoint("TOPLEFT", 3, -54)
	LFGShoutFrame.OptionsFrame.ChannelWorld:SetHitRectInsets(0, -110, 1, 5)
	LFGShoutFrame.OptionsFrame.ChannelWorld:SetChecked(LFGShout_Settings.ChannelWorld or false)
	LFGShoutFrame.OptionsFrame.ChannelWorld:SetScript("OnClick", 
		function(self, button)
			if self:GetChecked() then -- Autojoin World channel
				LFGShout_JoinChannel(WORLD_CHANNEL, WORLD_CHANNEL_NAME)
			end
			LFGShout_SaveSettings()
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
		end)

	-- Guild Checkbutton
	LFGShoutFrame.OptionsFrame.ChannelGuild = CreateFrame("CheckButton", nil, LFGShoutFrame.OptionsFrame, "InterfaceOptionsCheckButtonTemplate")
	LFGShoutFrame.OptionsFrame.ChannelGuild.Text:SetTextColor(1, 0.82, 0)
	LFGShoutFrame.OptionsFrame.ChannelGuild.Text:SetText("|c00FF5C1AG|r Guild Chat")
	LFGShoutFrame.OptionsFrame.ChannelGuild:SetPoint("TOPLEFT", 3, -72)
	LFGShoutFrame.OptionsFrame.ChannelGuild:SetHitRectInsets(0, -110, 1, 5)
	LFGShoutFrame.OptionsFrame.ChannelGuild:SetChecked(LFGShout_Settings.ChannelGuild or false)
	LFGShoutFrame.OptionsFrame.ChannelGuild:SetScript("OnClick", function(self, button) LFGShout_SaveSettings() PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON) end)

	-- SAY Checkbutton
	LFGShoutFrame.OptionsFrame.ChannelSay = CreateFrame("CheckButton", nil, LFGShoutFrame.OptionsFrame, "InterfaceOptionsCheckButtonTemplate")
	LFGShoutFrame.OptionsFrame.ChannelSay.Text:SetTextColor(1, 0.82, 0)
	LFGShoutFrame.OptionsFrame.ChannelSay.Text:SetText("|c00FF5C1AS|r Say")
	LFGShoutFrame.OptionsFrame.ChannelSay:SetPoint("TOPLEFT", 3, -90)
	LFGShoutFrame.OptionsFrame.ChannelSay:SetHitRectInsets(0, -110, 1, 5)
	LFGShoutFrame.OptionsFrame.ChannelSay:SetChecked(LFGShout_Settings.ChannelSay or false)
	LFGShoutFrame.OptionsFrame.ChannelSay:SetScript("OnClick", function(self, button) LFGShout_SaveSettings() PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON) end)

	-- YELL Checkbutton
	LFGShoutFrame.OptionsFrame.ChannelYell = CreateFrame("CheckButton", nil, LFGShoutFrame.OptionsFrame, "InterfaceOptionsCheckButtonTemplate")
	LFGShoutFrame.OptionsFrame.ChannelYell.Text:SetTextColor(1, 0.82, 0)
	LFGShoutFrame.OptionsFrame.ChannelYell.Text:SetText("|c00FF5C1AY|r Yell")
	LFGShoutFrame.OptionsFrame.ChannelYell:SetPoint("TOPLEFT", 3, -108)
	LFGShoutFrame.OptionsFrame.ChannelYell:SetHitRectInsets(0, -110, 1, 5)
	LFGShoutFrame.OptionsFrame.ChannelYell:SetChecked(LFGShout_Settings.ChannelYell or false)
	LFGShoutFrame.OptionsFrame.ChannelYell:SetScript("OnClick", function(self, button) LFGShout_SaveSettings() PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON) end)

	-- Textbox BEFORE
	LFGShoutFrame.OptionsFrame.StartText = CreateFrame("EditBox", nil, LFGShoutFrame.OptionsFrame)
	LFGShoutFrame.OptionsFrame.StartText:SetPoint("TOPLEFT", 3, -144)
	LFGShoutFrame.OptionsFrame.StartText:SetSize(139, 20)
	LFGShoutFrame.OptionsFrame.StartText:SetFontObject(GameFontWhite)
	LFGShoutFrame.OptionsFrame.StartText:SetTextInsets(4, 0, 0, 0) 
	LFGShoutFrame.OptionsFrame.StartText:SetBackdrop({
		bgFile="Interface/RaidFrame/UI-RaidFrame-GroupBg",
		edgeFile="Interface/Tooltips/UI-Tooltip-Border", 
		tile = false, tileSize = 0, edgeSize = 8,
		insets = { left = 2, right = 2, top = 2, bottom = 2 }
	})
	LFGShoutFrame.OptionsFrame.StartText:SetBackdropBorderColor(0.91, 0.91, 0.1, 0.5)
	LFGShoutFrame.OptionsFrame.StartText:SetText(LFGShout_Settings.StartText or "LFG")
	LFGShoutFrame.OptionsFrame.StartText:SetAutoFocus(false)

	LFGShoutFrame.OptionsFrame.StartText:SetScript("OnEscapePressed", function(self) self:ClearFocus() end)
	LFGShoutFrame.OptionsFrame.StartText:SetScript("OnEnterPressed", function(self) self:ClearFocus() end)
	LFGShoutFrame.OptionsFrame.StartText:SetScript("OnTabPressed", function(self) LFGShoutFrame.OptionsFrame.EndText:SetFocus() end)
	LFGShoutFrame.OptionsFrame.StartText:SetScript("OnTextChanged", LFGShout_SaveSettings)

	LFGShoutFrame.OptionsFrame.StartText.Label = LFGShoutFrame.OptionsFrame.StartText:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	LFGShoutFrame.OptionsFrame.StartText.Label:SetPoint("BOTTOMLEFT", LFGShoutFrame.OptionsFrame.StartText, "TOPLEFT", 2, 0)
	LFGShoutFrame.OptionsFrame.StartText.Label:SetText("Start Text")

	-- Textbox AFTER
	LFGShoutFrame.OptionsFrame.EndText = CreateFrame("EditBox", nil, LFGShoutFrame.OptionsFrame)
	LFGShoutFrame.OptionsFrame.EndText:SetPoint("TOPLEFT", 3, -180)
	LFGShoutFrame.OptionsFrame.EndText:SetSize(139, 20)
	LFGShoutFrame.OptionsFrame.EndText:SetFontObject(GameFontWhite)
	LFGShoutFrame.OptionsFrame.EndText:SetTextInsets(4, 0, 0, 0)
	LFGShoutFrame.OptionsFrame.EndText:SetBackdrop({
		bgFile="Interface/RaidFrame/UI-RaidFrame-GroupBg",
		edgeFile="Interface/Tooltips/UI-Tooltip-Border", 
		tile = false, tileSize = 0, edgeSize = 8,
		insets = { left = 2, right = 2, top = 2, bottom = 2 }
	})
	LFGShoutFrame.OptionsFrame.EndText:SetBackdropBorderColor(0.91, 0.91, 0.1, 0.5)
	LFGShoutFrame.OptionsFrame.EndText:SetText(LFGShout_Settings.EndText or "")
	LFGShoutFrame.OptionsFrame.EndText:SetAutoFocus(false)

	LFGShoutFrame.OptionsFrame.EndText:SetScript("OnEscapePressed", function(self) self:ClearFocus() end)
	LFGShoutFrame.OptionsFrame.EndText:SetScript("OnEnterPressed", function(self) self:ClearFocus() end)
	LFGShoutFrame.OptionsFrame.EndText:SetScript("OnTabPressed", function(self) LFGShoutFrame.OptionsFrame.StartText:SetFocus() end)
	LFGShoutFrame.OptionsFrame.EndText:SetScript("OnTextChanged", LFGShout_SaveSettings)
	
	LFGShoutFrame.OptionsFrame.EndText.Label = LFGShoutFrame.OptionsFrame.EndText:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	LFGShoutFrame.OptionsFrame.EndText.Label:SetPoint("BOTTOMLEFT", LFGShoutFrame.OptionsFrame.EndText, "TOPLEFT", 2, 0)
	LFGShoutFrame.OptionsFrame.EndText.Label:SetText("End Text")
	
	--[[
	-- Preset Dropdown
	local info = UIDropDownMenu_CreateInfo()
	LFGShoutFrame.OptionsFrame.Presets = CreateFrame("Frame", nil, LFGShoutFrame.OptionsFrame, "UIDropDownMenuTemplate")
	LFGShoutFrame.OptionsFrame.Presets:SetPoint("TOPLEFT", -15, -216)
	LFGShoutFrame.OptionsFrame.Presets:SetSize(10, 150)


--	LFGShoutFrame.OptionsFrame.Presets:SetScript("OnShow", 
--		function(self) 
--			DropDownMenu_SetSelectedValue(LFGShoutFrame.OptionsFrame.Presets, GetCVar("taintlog"))
--		end)
	LFGShoutFrame.OptionsFrame.Presets.initialize = 
		function()
			wipe(info)
			local values = { "0", "1", "2" }
			local tooltips = { "No Logging", "Light Logging", "Detailed Logging" }
			for i, value in pairs(values) do
				info.text = "Taintlog " .. value
				info.value = value
				info.tooltipTitle = info.text
				info.tooltipText = tooltips[i]
				info.tooltipOnButton = 1
				info.func =                 
					function(self)
						SetCVar("taintlog", self.value)
						print(format("|cffFF8800zToolz_Options >>|r taintlog = %s", GetCVar("taintlog")))
						DropDownMenu_SetSelectedValue(LFGShoutFrame.OptionsFrame.Presets, GetCVar("taintlog"))
					end
				info.checked = (value == GetCVar("taintlog"))
				UIDropDownMenu_AddButton(info)
			end
			UIDropDownMenu_SetSelectedValue(LFGShoutFrame.OptionsFrame.Presets, GetCVar("taintlog"))
--			DropDownList1:SetScale(1)
		end
	UIDropDownMenu_Initialize(LFGShoutFrame.OptionsFrame.Presets, LFGShoutFrame.OptionsFrame.Presets.initialize)
	UIDropDownMenu_JustifyText(LFGShoutFrame.OptionsFrame.Presets, "CENTER")

	LFGShoutFrame.OptionsFrame.Presets.Label = LFGShoutFrame.OptionsFrame.Presets:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	LFGShoutFrame.OptionsFrame.Presets.Label:SetPoint("BOTTOMLEFT", LFGShoutFrame.OptionsFrame.Presets, "TOPLEFT", 20, 0)
	LFGShoutFrame.OptionsFrame.Presets.Label:SetText("Presets")
	]]

	
	LFGShoutFrame:SetPoint("CENTER", LFGShoutFrame:GetParent(), "CENTER")

	LFGShout_Settings.AddonVisible = LFGShout_Settings.AddonVisible == nil and true or LFGShout_Settings.AddonVisible
	LFGShoutFrame:SetShown(LFGShout_Settings.AddonVisible)

	if LFGShout_Settings.MainFrameVisible == false then
		LFGShoutFrame:GetScript("OnMouseUp")(LFGShoutFrame, RIGHT_BUTTON)
	end

	if LFGShout_Settings.IconVisible == false then
		LFGShoutFrame:GetScript("OnMouseUp")(LFGShoutFrame, MIDDLE_BUTTON)
	end

	if LFGShout_Settings.OptionsFrameVisible == false then
		LFGShoutFrame.OptionsButton:GetScript("OnMouseUp")(LFGShoutFrame.OptionsButton)
	end

	if LFGShout_Settings.PrefixStar == nil then LFGShout_Settings.PrefixStar = true end

end

-- =============================================================================================================

local function LFGShoutFrame_OnEvent(self, event, ...)
	--print(event, time(), self:GetName(), ...)

	if event == "CHANNEL_UI_UPDATE" or event == "ZONE_CHANGED_NEW_AREA" then
		C_Timer.After(1, LFGShout_RefreshChannels)
		return
	end

	if event == "ADDON_LOADED" then
		if select(1, ...) == addonName then
			LFGShout_Load()
			LFGShout_RefreshChannels()
			self:UnregisterEvent("ADDON_LOADED")
		end
	end

end

-- =============================================================================================================

local function LFGShout_UnitPopup_ShowMenu(dropdownMenu, which, unit, name, userData)
	--print(dropdownMenu, which, unit, name, userData)
	--zd(dropdownMenu)
	--alerttable(dropdownMenu)
	--print(dropdownMenu:GetName())

	if string.find("FRIEND BN_FRIEND PARTY GUILD PLAYER FOCUS RAID_PLAYER COMMUNITIES_GUILD_MEMBER COMMUNITIES_WOW_MEMBER COMMUNITIES_MEMBER COMMUNITIES_COMMUNITY", dropdownMenu.which) then
	--		and (dropdownMenu.owner == nil) 
	--		if UIDROPDOWNMENU_MENU_LEVEL == 1 then

		UIDropDownMenu_AddSeparator();

		local info = UIDropDownMenu_CreateInfo()

		local text = ""
		
		table.foreach(LFGShout_Settings.CheckedDungeons, 
			function(i, id)
				text = format("%s / %s", text, DUNGEON_LIST[id].Code)
			end
		)

		info.func = function(self, name, role) SendChatMessage(format("Hi there :)  Want to %s %s ? %s", role, strtrim(text, " / "), LFGShoutFrame.OptionsFrame.EndText:GetText()), "WHISPER", nil, name) end
		info.owner = dropdownMenu
		info.arg1 = dropdownMenu.name
		info.notCheckable = true
		info.justifyH = "LEFT"
		
		local BUTTON_INFO = { "come", "Tank", "Heal", "DPS" }

		table.foreach(BUTTON_INFO, 
			function(k, role)
				info.arg2 = role
				info.text = format("|c00FF5C1ALFGShout|r <%s>", role)
				UIDropDownMenu_AddButton(info, 1)
			end
		)
		
	end
end

-- =============================================================================================================

local function LFGShout_Init()
	DEFAULT_CHAT_FRAME:AddMessage(format("[%s] |c00FFFFFFVersion:|r %s |c00FFFFFFLoaded|r", addonName, VERSION), 1, .48, 0)
	LFGShoutFrame:SetScript("OnEvent", LFGShoutFrame_OnEvent)
	LFGShoutFrame:RegisterEvent("ADDON_LOADED")
	LFGShoutFrame:RegisterEvent("CHANNEL_UI_UPDATE")
	LFGShoutFrame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	--LFGShoutFrame:RegisterAllEvents()
	hooksecurefunc("UnitPopup_ShowMenu", LFGShout_UnitPopup_ShowMenu)
	SLASH_LFGSHOUT1 = "/LFGS"
	function SlashCmdList.LFGSHOUT(msg)
		--message(msg)

		if string.lower(msg) == "star" then
			LFGShout_Settings.PrefixStar = not LFGShout_Settings.PrefixStar
			DEFAULT_CHAT_FRAME:AddMessage(format("|c00FF5C1ALFGShout: |rPrefix %sStar: %s", MARKER, LFGShout_Settings.PrefixStar and "Enabled" or "Disabled"))
		else
			PlaySoundFile("sound/interface/playerinvitea.ogg")
			LFGShoutFrame:SetShown(not LFGShoutFrame:IsShown())
			LFGShout_Settings.AddonVisible = LFGShoutFrame:IsShown()
		end
	end

	--[[
	C_Timer.After(2, 
		function()
--			LFGShout_JoinChannel(GENERAL_CHANNEL, GENERAL_CHANNEL_NAME)
--			LFGShout_JoinChannel(TRADE_CHANNEL, TRADE_CHANNEL_NAME)
			LFGShout_JoinChannel(LFG_CHANNEL, LFG_CHANNEL_NAME)
			LFGShout_JoinChannel(WORLD_CHANNEL, WORLD_CHANNEL_NAME)
		end
	)
	]]
end

-- =============================================================================================================

LFGShout_Init()

-- =============================================================================================================
